/*
  works together with the deviceConfigServer <-> producer Interface
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#ifndef __GENERICSTREAMPRODUCER_H
#define __GENERICSTREAMPRODUCER_H

#include <devices/streamProducer.h>
#include <producer/core/threadState.h>
#include <producer/core/deviceConfigServer.h>
#include <producer/core/producer.h>

/**
   A StreamProducer is the root device. It is the only device
   which _really_ produces data. All other devices which
   are connected to one another only depend on this
   device (more clearly: the data from it)
   <p>
   But although the StreamProducer has this meaningful task
   it's a very simple device, because there is nothing
   you can do with it (seen as a device). A StreamProducer
   is derived from an OutputDevice.(don't know why)
   <p>
   You attach a device with the setDevice Method to this producer.
   Then you can control wether the device is
   entered/not entered with the deviceEnter method.
   <p>
   This allows to generate:
   <p>
   *trees of devices with a StreamProdcuers at the root
   <p>
   *directed device graphs (with StreamProdcuers as a
    node with none incoming edges, and n outgoing edges.)
*/


class GenericStreamProducer : public StreamProducer, 
			             ThreadState ,
                                     DeviceConfigController {

  DeviceConfigServer* server;
  DeviceConfig* config;
  Producer* producer;

 public:
  GenericStreamProducer();
  ~GenericStreamProducer();


  int open(char* filename );
  int close();
  int play();
  int pause();
  int jump(int sec);

  int getState();     // returns the state of the thread


  void setProducer(Producer* producer);
  Producer* getProducer();

  // from ThreadState
  void setProducerEnter(int lServerEnter);
  void enqueueThreadState(int state);

  // must be public because of thread! never do call directly!
  void writeloop();

 private:
  void lockChangeMut();
  void unlockChangeMut();

  int lServerEnter;
  int lExit;
  int instanz;
  Buffer* lastFileName;

  pthread_t tr;

};
#endif
